"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var react_1 = __importDefault(require("react"));
var react_native_1 = require("react-native");
var utils_1 = require("../../../utils");
var split_change_1 = __importDefault(require("./split_change"));
var split_widget_1 = __importDefault(require("./split_widget"));
var keyForPair = function (x, y) {
    var keyForX = x ? utils_1.getChangeKey(x) : '00';
    var keyForY = y ? utils_1.getChangeKey(y) : '00';
    return keyForX + keyForY;
};
var groupElements = function (changes, widgets) {
    var findWidget = function (change) {
        if (!change) {
            return null;
        }
        var key = utils_1.getChangeKey(change);
        return widgets[key] || null;
    };
    var elements = [];
    // This could be a very complex reduce call, use `for` loop seems to make it a little more readable
    for (var i = 0; i < changes.length; i++) {
        var current = changes[i];
        // A normal change is displayed on both side
        if (current.isNormal) {
            elements.push(['change', keyForPair(current, current), current, current]);
        }
        else if (current.isDelete) {
            var next = changes[i + 1];
            // If an insert change is following a elete change, they should be displayed side by side
            if (next && next.isInsert) {
                i = i + 1;
                elements.push(['change', keyForPair(current, next), current, next]);
            }
            else {
                elements.push(['change', keyForPair(current, null), current, null]);
            }
        }
        else {
            elements.push(['change', keyForPair(null, current), null, current]);
        }
        var rowChanges = elements[elements.length - 1];
        var _a = [rowChanges[2], rowChanges[3]].map(function (change) { return findWidget(change); }), oldWidget = _a[0], newWidget = _a[1];
        if (oldWidget || newWidget) {
            var key = rowChanges[1];
            elements.push(['widget', key, oldWidget, newWidget]);
        }
    }
    return elements;
};
var renderRow = function (_a, props) {
    var type = _a[0], key = _a[1], oldValue = _a[2], newValue = _a[3];
    var selectedChanges = props.selectedChanges, monotonous = props.monotonous, hideGutter = props.hideGutter, tokens = props.tokens, lineStyle = props.lineStyle, changeProps = __rest(props, ["selectedChanges", "monotonous", "hideGutter", "tokens", "lineStyle"]);
    if (type === 'change') {
        var oldSelected = oldValue ? selectedChanges.includes(utils_1.getChangeKey(oldValue)) : false;
        var newSelected = newValue ? selectedChanges.includes(utils_1.getChangeKey(newValue)) : false;
        var oldTokens = oldValue && tokens ? tokens.old[utils_1.computeOldLineNumber(oldValue) - 1] : null;
        var newTokens = newValue && tokens ? tokens.new[utils_1.computeNewLineNumber(newValue) - 1] : null;
        return (<split_change_1.default key={"change" + key} style={lineStyle} oldChange={oldValue} newChange={newValue} monotonous={monotonous} hideGutter={hideGutter} oldSelected={oldSelected} newSelected={newSelected} oldTokens={oldTokens} newTokens={newTokens} {...changeProps}/>);
    }
    else if (type === 'widget') {
        return (<split_widget_1.default key={"widget" + key} monotonous={monotonous} oldElement={oldValue} newElement={newValue}/>);
    }
    return null;
};
var SplitHunk = react_1.default.memo(function (props) {
    var hunk = props.hunk, widgets = props.widgets, style = props.style, childrenProps = __rest(props, ["hunk", "widgets", "style"]);
    var elements = groupElements(hunk.changes, widgets);
    return <react_native_1.View style={style}>{elements.map(function (item) { return renderRow(item, childrenProps); })}</react_native_1.View>;
});
exports.default = SplitHunk;
//# sourceMappingURL=index.js.map