"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var react_1 = __importStar(require("react"));
var react_native_1 = require("react-native");
var utils_1 = require("../utils");
var classnames_1 = __importDefault(require("../../../utils/classnames"));
var code_cell_1 = __importDefault(require("../code_cell"));
var styles_1 = __importDefault(require("../styles"));
var useCallbackOnSide = function (side, setHover, change, customCallbacks) {
    var markHover = react_1.useCallback(function () { return setHover(side); }, [side, setHover]);
    var unmarkHover = react_1.useCallback(function () { return setHover(null); }, [setHover]);
    var arg = { side: side, change: change };
    // Unlike selectors, hooks do not provide native functionality to customize comparator,
    // on realizing that this does not reduce amount of renders, only preventing duplicate merge computations,
    // we decide not to optimize this extremely, leave it recomputed on certain rerenders.
    var callbacks = react_1.useMemo(function () {
        var output = Object.keys(callbacks).reduce(function (acc, key) {
            acc[key] = function (fn) { return function () { return fn(arg); }; };
            return acc;
        }, {});
        output.onMouseEnter = utils_1.composeCallback(markHover, output.onMouseEnter);
        output.onMouseLeave = utils_1.composeCallback(unmarkHover, output.onMouseLeave);
        return output;
    }, [change, customCallbacks, markHover, unmarkHover]);
    return callbacks;
};
var renderCells = function (args) {
    var change = args.change, side = args.side, tokens = args.tokens, gutterStyle = args.gutterStyle, codeStyle = args.codeStyle, gutterEvents = args.gutterEvents, codeEvents = args.codeEvents, hideGutter = args.hideGutter, hover = args.hover, renderToken = args.renderToken, renderGutter = args.renderGutter, onChangePress = args.onChangePress;
    var computedGutterStyle = classnames_1.default(gutterStyle, [styles_1.default.diffGutterInsert, change.type === 'insert'], [styles_1.default.diffGutterDelete, change.type === 'delete']);
    var computedCodeStyle = classnames_1.default(codeStyle, [styles_1.default.diffCodeCellInsert, change.type === 'insert'], [styles_1.default.diffCodeCellDelete, change.type === 'delete']);
    if (!change) {
        return (<react_1.default.Fragment>
        {!hideGutter && <react_native_1.View key="gutter" style={computedGutterStyle}/>}
        <react_native_1.View key="code" style={codeStyle}/>
      </react_1.default.Fragment>);
    }
    var content = change.content;
    var gutterOptions = {
        change: change,
        side: side,
        inHoverState: hover,
        renderDefault: utils_1.renderDefaultBy(change, side),
    };
    var gutterProps = __assign({ style: gutterStyle, children: renderGutter(gutterOptions) }, gutterEvents);
    return (<react_native_1.TouchableOpacity disabled={!onChangePress} activeOpacity={0.65} onPress={function () { return onChangePress(change); }}>
      {!hideGutter && <react_native_1.View key="gutter" {...gutterProps}/>}
      <code_cell_1.default key="code" style={computedCodeStyle} text={content} tokens={tokens} renderToken={renderToken} {...codeEvents}/>
    </react_native_1.TouchableOpacity>);
};
var SplitChange = react_1.default.memo(function (props) {
    var style = props.style, gutterStyle = props.gutterStyle, codeStyle = props.codeStyle, gutterEvents = props.gutterEvents, codeEvents = props.codeEvents, oldChange = props.oldChange, newChange = props.newChange, oldSelected = props.oldSelected, newSelected = props.newSelected, oldTokens = props.oldTokens, newTokens = props.newTokens, monotonous = props.monotonous, hideGutter = props.hideGutter, renderToken = props.renderToken, renderGutter = props.renderGutter;
    var _a = react_1.useState(''), hover = _a[0], setHover = _a[1];
    var oldGutterEvents = useCallbackOnSide('old', setHover, oldChange, gutterEvents);
    var newGutterEvents = useCallbackOnSide('new', setHover, newChange, gutterEvents);
    var oldCodeEvents = useCallbackOnSide('old', setHover, oldChange, codeEvents);
    var newCodeEvents = useCallbackOnSide('new', setHover, newChange, codeEvents);
    var commons = {
        monotonous: monotonous,
        hideGutter: hideGutter,
        gutterStyle: gutterStyle,
        codeStyle: codeStyle,
        gutterEvents: gutterEvents,
        codeEvents: codeEvents,
        renderToken: renderToken,
        renderGutter: renderGutter,
    };
    var oldArgs = __assign(__assign({}, commons), { change: oldChange, side: 'old', selected: oldSelected, tokens: oldTokens, gutterEvents: oldGutterEvents, codeEvents: oldCodeEvents, hover: hover === 'old' });
    var newArgs = __assign(__assign({}, commons), { change: newChange, side: 'new', selected: newSelected, tokens: newTokens, gutterEvents: newGutterEvents, codeEvents: newCodeEvents, hover: hover === 'new' });
    if (monotonous) {
        return <react_native_1.View style={style}>{renderCells(oldChange ? oldArgs : newArgs)}</react_native_1.View>;
    }
    return (<react_native_1.View style={style}>
        {renderCells(oldArgs)}
        {renderCells(newArgs)}
      </react_native_1.View>);
});
SplitChange.defaultProps = {
    oldTokens: null,
    newTokens: null,
};
exports.default = SplitChange;
//# sourceMappingURL=split_change.js.map