"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var react_1 = __importStar(require("react"));
var react_native_1 = require("react-native");
var utils_1 = require("../utils");
var classnames_1 = __importDefault(require("../../../utils/classnames"));
var code_cell_1 = __importDefault(require("../code_cell"));
var styles_1 = __importDefault(require("../styles"));
var useBoundCallbacks = function (callbacks, arg, hoverOn, hoverOff) {
    return react_1.useMemo(function () {
        var output = Object.keys(callbacks).reduce(function (acc, key) {
            acc[key] = function (fn) { return function () { return fn(arg); }; };
            return acc;
        }, {});
        output.onMouseEnter = utils_1.composeCallback(hoverOn, output.onMouseEnter);
        output.onMouseLeave = utils_1.composeCallback(hoverOff, output.onMouseLeave);
        return output;
    }, [callbacks, arg, hoverOn, hoverOff]);
};
var useBoolean = function () {
    var _a = react_1.useState(false), value = _a[0], setValue = _a[1];
    var on = react_1.useCallback(function () { return setValue(true); }, []);
    var off = react_1.useCallback(function () { return setValue(false); }, []);
    return [value, on, off];
};
var renderGutterCell = function (style, change, side, events, inHoverState, renderGutter) {
    var gutterOptions = {
        change: change,
        side: side,
        inHoverState: inHoverState,
        textStyle: styles_1.default.diffGutterText,
        renderDefault: utils_1.renderDefaultBy(change, side),
    };
    return (<react_native_1.View style={classnames_1.default(styles_1.default.diffGutter, style)} {...events}>
      {renderGutter(gutterOptions)}
    </react_native_1.View>);
};
var UnifiedChange = react_1.default.memo(function (props) {
    var change = props.change, tokens = props.tokens, style = props.style, gutterStyle = props.gutterStyle, codeStyle = props.codeStyle, gutterEvents = props.gutterEvents, codeEvents = props.codeEvents, hideGutter = props.hideGutter, renderToken = props.renderToken, renderGutter = props.renderGutter, onChangePress = props.onChangePress;
    var content = change.content;
    var _a = useBoolean(), hover = _a[0], hoverOn = _a[1], hoverOff = _a[2];
    var eventArg = react_1.useMemo(function () { return ({ change: change }); }, [change]);
    var boundGutterEvents = useBoundCallbacks(gutterEvents, eventArg, hoverOn, hoverOff);
    var boundCodeEvents = useBoundCallbacks(codeEvents, eventArg, hoverOn, hoverOff);
    var computedGutterStyle = classnames_1.default(gutterStyle, [styles_1.default.diffGutterInsert, change.type === 'insert'], [styles_1.default.diffGutterDelete, change.type === 'delete']);
    var computedCodeStyle = classnames_1.default(codeStyle, [styles_1.default.diffCodeCellInsert, change.type === 'insert'], [styles_1.default.diffCodeCellDelete, change.type === 'delete']);
    return (<react_native_1.TouchableOpacity style={classnames_1.default(styles_1.default.diffUnifiedChange, style)} disabled={!onChangePress} activeOpacity={0.65} onPress={function () { return onChangePress(change); }}>
      {!hideGutter && renderGutterCell(computedGutterStyle, change, 'old', boundGutterEvents, hover, renderGutter)}
      {!hideGutter && renderGutterCell(computedGutterStyle, change, 'new', boundGutterEvents, hover, renderGutter)}
      <code_cell_1.default style={computedCodeStyle} text={content} tokens={tokens} renderToken={renderToken} {...boundCodeEvents}/>
    </react_native_1.TouchableOpacity>);
});
UnifiedChange.defaultProps = {
    tokens: null,
};
exports.default = UnifiedChange;
//# sourceMappingURL=unified_change.js.map