"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var diff_match_patch_1 = require("diff-match-patch");
var pick_ranges_1 = __importDefault(require("./pick_ranges"));
var util_1 = require("../utils/diff/util");
var findChangeBlocks = function (changes) {
    var start = changes.findIndex(function (change) { return !change.isNormal; });
    if (start === -1) {
        return [];
    }
    var end = __spreadArrays(changes.slice(start)).findIndex(function (change) { return change.isNormal; });
    if (end === -1) {
        return [changes.slice(start)];
    }
    return __spreadArrays([changes.slice(start, end)], findChangeBlocks(changes.slice(end)));
};
var groupDiffs = function (diffs) {
    return diffs.reduce(function (_a, diff) {
        var oldDiffs = _a[0], newDiffs = _a[1];
        var type = diff[0];
        switch (type) {
            case diff_match_patch_1.DIFF_INSERT:
                newDiffs.push(diff);
                break;
            case diff_match_patch_1.DIFF_DELETE:
                oldDiffs.push(diff);
                break;
            default:
                oldDiffs.push(diff);
                newDiffs.push(diff);
                break;
        }
        return [oldDiffs, newDiffs];
    }, [[], []]);
};
var splitDiffToLines = function (diffs) {
    return diffs.reduce(function (lines, _a) {
        var type = _a[0], value = _a[1];
        var currentLines = value.split('\n');
        var _b = currentLines.map(function (line) { return [type, line]; }), currentLineRemaining = _b[0], nextLines = _b.slice(1);
        var next = __spreadArrays(lines.slice(0, -1), [__spreadArrays(util_1.last(lines), [currentLineRemaining])], nextLines.map(function (line) { return [line]; }));
        return next;
    }, [[]]);
};
var diffsToEdits = function (diffs, lineNumber) {
    var output = diffs.reduce(function (acc, diff) {
        var edits = acc.edits, start = acc.start;
        var type = diff[0], value = diff[1];
        if (type !== diff_match_patch_1.DIFF_EQUAL) {
            var edit = {
                type: 'edit',
                lineNumber: lineNumber,
                start: start,
                length: value.length,
            };
            edits.push(edit);
        }
        return {
            edits: edits,
            start: start + value.length,
        };
    }, { edits: [], start: 0 });
    return output.edits;
};
var convertToLinesOfEdits = function (linesOfDiffs, startLineNumber) {
    return linesOfDiffs.reduce(function (acc, diffs, index) {
        acc.push(diffsToEdits(diffs, startLineNumber + index));
        return acc;
    }, []);
};
var diffText = function (a, b) {
    var dmp = new diff_match_patch_1.diff_match_patch();
    var diffs = dmp.diff_main(a, b);
    dmp.diff_cleanupSemantic(diffs);
    if (diffs.length <= 1) {
        return [[], []];
    }
    return groupDiffs(diffs);
};
var diffChangeBlock = function (changes) {
    var _a = changes.reduce(function (_a, _b) {
        var oldSource = _a[0], newSource = _a[1];
        var isDelete = _b.isDelete, content = _b.content;
        return isDelete
            ? [oldSource + (oldSource ? '\n' : '') + content, newSource]
            : [oldSource, newSource + (newSource ? '\n' : '') + content];
    }, ['', '']), oldSource = _a[0], newSource = _a[1];
    var _b = diffText(oldSource, newSource), oldDiffs = _b[0], newDiffs = _b[1];
    if (oldDiffs.length === 0 && newDiffs.length === 0) {
        return {
            oldEdits: oldDiffs,
            newEdits: newDiffs,
        };
    }
    var getLineNumber = function (change) { return change && change.lineNumber; };
    var oldStartLineNumber = getLineNumber(changes.find(function (change) { return change.isDelete; }));
    var newStartLineNumber = getLineNumber(changes.find(function (change) { return change.isInsert; }));
    var oldEdits = convertToLinesOfEdits(splitDiffToLines(oldDiffs), oldStartLineNumber);
    var newEdits = convertToLinesOfEdits(splitDiffToLines(newDiffs), newStartLineNumber);
    return {
        oldEdits: oldEdits,
        newEdits: newEdits,
    };
};
var diffByLine = function (changes) {
    return changes.reduce(function (_a, currentChange) {
        var oldEdits = _a.oldEdits, newEdits = _a.newEdits, previousChange = _a.previousChange;
        if (!previousChange.isDelete || !currentChange.isInsert) {
            return { oldEdits: oldEdits, newEdits: newEdits, previousChange: currentChange };
        }
        var _b = diffText(previousChange.content, currentChange.content), oldDiffs = _b[0], newDiffs = _b[1];
        return {
            oldEdits: oldEdits.concat(diffsToEdits(oldDiffs, previousChange.lineNumber)),
            newEdits: newEdits.concat(diffsToEdits(newDiffs, currentChange.lineNumber)),
            previousChange: currentChange,
        };
    }, {
        oldEdits: [],
        newEdits: [],
        previousChange: {},
    });
};
exports.default = (function (hunks, _a) {
    var _b = (_a === void 0 ? {} : _a).type, type = _b === void 0 ? 'block' : _b;
    var findEdits = type === 'block' ? diffChangeBlock : diffByLine;
    var changeBlocks = hunks.reduce(function (acc, hunk) {
        acc.push.apply(acc, findChangeBlocks(hunk.changes));
        return acc;
    }, []);
    var _c = changeBlocks
        .map(function (edits) { return findEdits(edits); })
        .reduce(function (_a, _b) {
        var oldEdits = _a.oldEdits, newEdits = _a.newEdits;
        var currentOld = _b.oldEdits, currentNew = _b.newEdits;
        return {
            oldEdits: oldEdits.concat(currentOld),
            newEdits: newEdits.concat(currentNew),
        };
    }, { oldEdits: [], newEdits: [] }), oldEdits = _c.oldEdits, newEdits = _c.newEdits;
    return pick_ranges_1.default(util_1.flatten(oldEdits), util_1.flatten(newEdits));
});
//# sourceMappingURL=mark_edits.js.map