"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
var insert_hunk_1 = require("./insert_hunk");
var factory_1 = require("./factory");
var util_1 = require("./util");
var getCorrespondingNewLineNumber = factory_1.getCorrespondingLineNumberFactory('old');
var computeOldLineNumber = factory_1.computeLineNumberFactory('old');
var isOldLineNumberInHunk = factory_1.isInHunkFactory('oldStart', 'oldLines');
var isOldLineNumberBetweenHunks = factory_1.isBetweenHunksFactory('oldStart', 'oldLines');
var findCorrespondingValidHunkIndex = function (hunks, oldLineNumber) {
    if (!hunks.length) {
        return -1;
    }
    var firstHunk = util_1.first(hunks);
    if (oldLineNumber < firstHunk.oldStart || isOldLineNumberInHunk(firstHunk, oldLineNumber)) {
        return 0;
    }
    for (var i = 1; i < hunks.length; i++) {
        var currentHunk = hunks[i];
        if (isOldLineNumberInHunk(currentHunk, oldLineNumber)) {
            return i;
        }
        var previousHunk = hunks[i - 1];
        if (isOldLineNumberBetweenHunks(previousHunk, currentHunk, oldLineNumber)) {
            return i;
        }
    }
    return -1;
};
var findNearestNormalChangeIndex = function (_a, start) {
    var changes = _a.changes;
    var index = changes.findIndex(function (change) { return computeOldLineNumber(change) === start; });
    for (var i = index; i < changes.length; i++) {
        var change = changes[i];
        if (change.isNormal) {
            return i;
        }
    }
    return -1;
};
var splitRangeToValidOnes = function (hunks, start, end) {
    var correspondingHunkIndex = findCorrespondingValidHunkIndex(hunks, start);
    // `start` is after all hunks, we believe all left lines are normal.
    if (correspondingHunkIndex === -1) {
        return [[start, end]];
    }
    var correspondingHunk = hunks[correspondingHunkIndex];
    // If `start` points to a line before this hunk, we collect all heading normal changes
    if (start < correspondingHunk.oldStart) {
        var headingChangesCount = correspondingHunk.changes.findIndex(function (change) { return !change.isNormal; });
        var validEnd_1 = correspondingHunk.oldStart + Math.max(headingChangesCount, 0);
        if (validEnd_1 >= end) {
            return [[start, end]];
        }
        return __spreadArrays([[start, validEnd_1]], splitRangeToValidOnes(hunks, validEnd_1 + 1, end));
    }
    // Now the `correspondingHunk` must be a hunk containing `start`,
    // however it is still possible that `start` is not a normal change
    var changes = correspondingHunk.changes;
    var nearestNormalChangeIndex = findNearestNormalChangeIndex(correspondingHunk, start);
    // If there is no normal changes after `start`, splitting ends up here
    if (nearestNormalChangeIndex === -1) {
        return [];
    }
    var validStartChange = changes[nearestNormalChangeIndex];
    var validStart = computeOldLineNumber(validStartChange);
    // Iterate to `end`, if `end` falls out of hunk, we can split it to 2 ranges
    var adjacentChangesCount = changes.slice(nearestNormalChangeIndex + 1).findIndex(function (change) { return !change.isNormal; });
    var validEnd = computeOldLineNumber(validStartChange) + Math.max(adjacentChangesCount, 0);
    if (validEnd >= end) {
        return [[validStart, end]];
    }
    return __spreadArrays([[validStart, validEnd]], splitRangeToValidOnes(hunks, validEnd + 1, end));
};
var expandCodeByValidRange = function (hunks, rawCodeOrLines, _a) {
    var start = _a[0], end = _a[1];
    // Note `end` is not inclusive, this is the same as `Array.prototype.slice` method
    var linesOfCode = typeof rawCodeOrLines === 'string' ? rawCodeOrLines.split('\n') : rawCodeOrLines;
    var slicedLines = linesOfCode.slice(Math.max(start, 1) - 1, end - 1);
    if (!slicedLines.length) {
        return hunks;
    }
    var slicedHunk = insert_hunk_1.textLinesToHunk(slicedLines, start, getCorrespondingNewLineNumber(hunks, start));
    return insert_hunk_1.insertHunk(hunks, slicedHunk);
};
exports.expandFromRawCode = function (hunks, rawCodeOrLines, start, end) {
    // It is possible to have some insert or delete changes between `start` and `end`,
    // in order to be 100% safe, we need to split the range to one or more ranges which contains only normal changes.
    //
    // For each `start` line number, we can either:
    //
    // 1. Find a change and adjust to a nearest normal one.
    // 2. Find no corresponding change so it must be a collapsed normal change.
    //
    // For both cases we can have a starting normal change, then we iterate over its subsequent changes
    // (line numbers with no corresponding change is considered a normal one)
    // until an insert or delete is encountered, this is a **valid range**.
    //
    // After one valid range is resolved, discard all line numbers related to delete changes, the next normal change
    // is the start of next valid range.
    var validRanges = splitRangeToValidOnes(hunks, start, end);
    return validRanges.reduce(function (hunks, range) { return expandCodeByValidRange(hunks, rawCodeOrLines, range); }, hunks);
};
exports.getCollapsedLinesCountBetween = function (previousHunk, nextHunk) {
    if (!previousHunk) {
        return nextHunk.oldStart - 1;
    }
    if (!nextHunk) {
        throw new Error('Unable to compute lines count after the last hunk');
    }
    var previousEnd = previousHunk.oldStart + previousHunk.oldLines;
    var nextStart = nextHunk.oldStart;
    return nextStart - previousEnd;
};
exports.expandCollapsedBlockBy = function (hunks, rawCodeOrLines, predicate) {
    var linesOfCode = typeof rawCodeOrLines === 'string' ? rawCodeOrLines.split('\n') : rawCodeOrLines;
    var firstHunk = util_1.first(hunks);
    var initialExpandingBlocks = predicate(firstHunk.oldStart - 1, 1, 1) ? [[1, firstHunk.oldStart]] : [];
    var expandingBlocks = hunks.reduce(function (expandingBlocks, currentHunk, index, hunks) {
        var nextHunk = hunks[index + 1];
        var oldStart = currentHunk.oldStart + currentHunk.oldLines;
        var newStart = currentHunk.newStart + currentHunk.newLines;
        var lines = nextHunk ? exports.getCollapsedLinesCountBetween(currentHunk, nextHunk) : linesOfCode.length - oldStart + 1;
        var shouldExpand = predicate(lines, oldStart, newStart);
        return shouldExpand ? __spreadArrays(expandingBlocks, [[oldStart, oldStart + lines]]) : expandingBlocks;
    }, initialExpandingBlocks);
    return expandingBlocks.reduce(function (hunks, _a) {
        var start = _a[0], end = _a[1];
        return exports.expandFromRawCode(hunks, linesOfCode, start, end);
    }, hunks);
};
//# sourceMappingURL=expand_collapsed_block_by.js.map