"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var util_1 = require("./util");
exports.computeLineNumberFactory = function (side) {
    if (side === 'old') {
        return function (_a) {
            var isNormal = _a.isNormal, isInsert = _a.isInsert, lineNumber = _a.lineNumber, oldLineNumber = _a.oldLineNumber;
            if (isInsert) {
                return -1;
            }
            return isNormal ? oldLineNumber : lineNumber;
        };
    }
    return function (_a) {
        var isNormal = _a.isNormal, isDelete = _a.isDelete, lineNumber = _a.lineNumber, newLineNumber = _a.newLineNumber;
        if (isDelete) {
            return -1;
        }
        return isNormal ? newLineNumber : lineNumber;
    };
};
exports.isInHunkFactory = function (startProperty, linesProperty) { return function (hunk, lineNumber) {
    var start = hunk[startProperty];
    var end = start + hunk[linesProperty];
    return lineNumber >= start && lineNumber < end;
}; };
exports.isBetweenHunksFactory = function (startProperty, linesProperty) { return function (previousHunk, nextHunk, lineNumber) {
    var start = previousHunk[startProperty] + previousHunk[linesProperty];
    var end = nextHunk[startProperty];
    return lineNumber >= start && lineNumber < end;
}; };
var findContainerHunkFactory = function (side) {
    var _a = util_1.sideToProperty(side), startProperty = _a[0], linesProperty = _a[1];
    var isInHunk = exports.isInHunkFactory(startProperty, linesProperty);
    return function (hunks, lineNumber) { return hunks.find(function (hunk) { return isInHunk(hunk, lineNumber); }); };
};
exports.findChangeByLineNumberFactory = function (side) {
    var computeLineNumber = exports.computeLineNumberFactory(side);
    var findContainerHunk = findContainerHunkFactory(side);
    return function (hunks, lineNumber) {
        var containerHunk = findContainerHunk(hunks, lineNumber);
        if (!containerHunk) {
            return undefined;
        }
        return containerHunk.changes.find(function (change) { return computeLineNumber(change) === lineNumber; });
    };
};
exports.getCorrespondingLineNumberFactory = function (baseSide) {
    var anotherSide = baseSide === 'old' ? 'new' : 'old';
    var _a = util_1.sideToProperty(baseSide), baseStart = _a[0], baseLines = _a[1];
    var _b = util_1.sideToProperty(anotherSide), correspondingStart = _b[0], correspondingLines = _b[1];
    var baseLineNumber = exports.computeLineNumberFactory(baseSide);
    var correspondingLineNumber = exports.computeLineNumberFactory(anotherSide);
    var isInHunk = exports.isInHunkFactory(baseStart, baseLines);
    var isBetweenHunks = exports.isBetweenHunksFactory(baseStart, baseLines);
    return function (hunks, lineNumber) {
        var firstHunk = util_1.first(hunks);
        // Before first hunk
        if (lineNumber < firstHunk[baseStart]) {
            var spanFromStart = firstHunk[baseStart] - lineNumber;
            return firstHunk[correspondingStart] - spanFromStart;
        }
        // After last hunk, this can be done in `for` loop, just a quick return path
        var lastHunk = util_1.last(hunks);
        if (lastHunk[baseStart] + lastHunk[baseLines] <= lineNumber) {
            var spanFromEnd = lineNumber - lastHunk[baseStart] - lastHunk[baseLines];
            return lastHunk[correspondingStart] + lastHunk[correspondingLines] + spanFromEnd;
        }
        for (var i = 0; i < hunks.length; i++) {
            var currentHunk = hunks[i];
            var nextHunk = hunks[i + 1];
            // Within current hunk
            if (isInHunk(currentHunk, lineNumber)) {
                var changeIndex = currentHunk.changes.findIndex(function (change) { return baseLineNumber(change) === lineNumber; });
                var change = currentHunk.changes[changeIndex];
                if (change.isNormal) {
                    return correspondingLineNumber(change);
                }
                // For changes of type "insert" and "delete", the sibling change can be the corresponding one,
                // or they can have no corresponding change
                //
                // Git diff always put delete change before insert change
                //
                // Note that `nearbySequences: "zip"` option can affect this function
                var possibleCorrespondingChangeIndex = change.isDelete ? changeIndex + 1 : changeIndex - 1;
                var possibleCorrespondingChange = currentHunk.changes[possibleCorrespondingChangeIndex];
                if (!possibleCorrespondingChange) {
                    return -1;
                }
                var negativeChangeType = change.isInsert ? 'delete' : 'insert';
                return possibleCorrespondingChange.type === negativeChangeType
                    ? correspondingLineNumber(possibleCorrespondingChange)
                    : -1;
            }
            // Between 2 hunks
            if (isBetweenHunks(currentHunk, nextHunk, lineNumber)) {
                var spanFromEnd = lineNumber - currentHunk[baseStart] - currentHunk[baseLines];
                return currentHunk[correspondingStart] + currentHunk[correspondingLines] + spanFromEnd;
            }
        }
        throw new Error("Unexpected line position " + lineNumber);
    };
};
//# sourceMappingURL=factory.js.map