"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
var factory_1 = require("./factory");
var util_1 = require("./util");
var computeOldLineNumber = factory_1.computeLineNumberFactory('old');
var computeNewLineNumber = factory_1.computeLineNumberFactory('new');
var getOldRangeFromHunk = function (_a) {
    var oldStart = _a.oldStart, oldLines = _a.oldLines;
    return [oldStart, oldStart + oldLines - 1];
};
var createHunkFromChanges = function (changes) {
    if (!changes.length) {
        return null;
    }
    var initial = {
        isPlain: true,
        content: '',
        oldStart: -1,
        oldLines: 0,
        newStart: -1,
        newLines: 0,
    };
    var hunk = changes.reduce(function (hunk, change) {
        if (!change.isNormal) {
            hunk.isPlain = false;
        }
        if (!change.isInsert) {
            hunk.oldLines = hunk.oldLines + 1;
            if (hunk.oldStart === -1) {
                hunk.oldStart = computeOldLineNumber(change);
            }
        }
        if (!change.isDelete) {
            hunk.newLines = hunk.newLines + 1;
            if (hunk.newStart === -1) {
                hunk.newStart = computeNewLineNumber(change);
            }
        }
        return hunk;
    }, initial);
    var oldStart = hunk.oldStart, oldLines = hunk.oldLines, newStart = hunk.newStart, newLines = hunk.newLines;
    return __assign(__assign({}, hunk), { content: "@@ -" + oldStart + "," + oldLines + " +" + newStart + "," + newLines, changes: changes });
};
exports.textLinesToHunk = function (lines, oldStartLineNumber, newStartLineNumber) {
    var lineToChange = function (line, index) {
        return {
            type: 'normal',
            isNormal: true,
            oldLineNumber: oldStartLineNumber + index,
            newLineNumber: newStartLineNumber + index,
            content: '' + line,
        };
    };
    var changes = lines.map(lineToChange);
    return createHunkFromChanges(changes);
};
var sliceHunk = function (_a, startOldLineNumber, endOldLineNumber) {
    var changes = _a.changes;
    var changeIndex = changes.findIndex(function (change) { return computeOldLineNumber(change) >= startOldLineNumber; });
    if (changeIndex === -1) {
        return null;
    }
    // It is possible to have some insert changes before `startOldLineNumber`,
    // since we slice from old line number, these changes can be ommited, so we need to grab them back
    var startIndex = (function () {
        if (changeIndex === 0) {
            return changeIndex;
        }
        var nearestHeadingNormalChangeIndex = util_1.findLastIndex(changes, function (change) { return !change.isInsert; }, changeIndex - 1);
        return nearestHeadingNormalChangeIndex === -1 ? changeIndex : nearestHeadingNormalChangeIndex + 1;
    })();
    if (endOldLineNumber === undefined) {
        var slicedChanges_1 = changes.slice(startIndex);
        return createHunkFromChanges(slicedChanges_1);
    }
    var endIndex = util_1.findLastIndex(changes, function (change) { return computeOldLineNumber(change) <= endOldLineNumber; });
    var slicedChanges = changes.slice(startIndex, endIndex === -1 ? undefined : endIndex);
    return createHunkFromChanges(slicedChanges);
};
var mergeHunk = function (previousHunk, nextHunk) {
    if (!previousHunk) {
        return nextHunk;
    }
    if (!nextHunk) {
        return previousHunk;
    }
    var _a = getOldRangeFromHunk(previousHunk), previousStart = _a[0], previousEnd = _a[1];
    var _b = getOldRangeFromHunk(nextHunk), nextStart = _b[0], nextEnd = _b[1];
    // They are just neighboring, simply concat changes and adjust lines count
    if (previousEnd + 1 === nextStart) {
        return createHunkFromChanges(__spreadArrays(previousHunk.changes, nextHunk.changes));
    }
    // It is possible that `previousHunk` entirely **contains** `nextHunk`,
    // and if we are merging a fake hunk with a valid hunk, we need to replace `nextHunk`'s corresponding range
    if (previousStart <= nextStart && previousEnd >= nextEnd) {
        if (previousHunk.isPlain && !nextHunk.isPlain) {
            var head = sliceHunk(previousHunk, previousStart, nextStart);
            var tail_1 = sliceHunk(previousHunk, nextEnd + 1);
            return mergeHunk(mergeHunk(head, nextHunk), tail_1);
        }
        return previousHunk;
    }
    // The 2 hunks have some overlapping, we need to slice the fake one in order to preserve non-normal changes
    if (previousHunk.isPlain) {
        var head = sliceHunk(previousHunk, previousStart, nextStart);
        return mergeHunk(head, nextHunk);
    }
    var tail = sliceHunk(nextHunk, previousEnd + 1);
    return mergeHunk(previousHunk, tail);
};
var appendOrMergeHunk = function (hunks, nextHunk) {
    var lastHunk = util_1.last(hunks);
    if (!lastHunk) {
        return [nextHunk];
    }
    var expectedNextStart = lastHunk.oldStart + lastHunk.oldLines;
    var actualNextStart = nextHunk.oldStart;
    if (expectedNextStart < actualNextStart) {
        return hunks.concat(nextHunk);
    }
    var mergedHunk = mergeHunk(lastHunk, nextHunk);
    return __spreadArrays(hunks.slice(0, -1), [mergedHunk]);
};
exports.insertHunk = function (hunks, insertion) {
    var insertionOldLineNumber = computeOldLineNumber(insertion.changes[0]);
    var isLaterThanInsertion = function (_a) {
        var changes = _a.changes;
        if (!changes.length) {
            return false;
        }
        return computeOldLineNumber(changes[0]) >= insertionOldLineNumber;
    };
    var insertPosition = hunks.findIndex(isLaterThanInsertion);
    var hunksWithInsertion = insertPosition === -1
        ? hunks.concat(insertion)
        : __spreadArrays(hunks.slice(0, insertPosition), [insertion], hunks.slice(insertPosition));
    return hunksWithInsertion.reduce(appendOrMergeHunk, []);
};
//# sourceMappingURL=insert_hunk.js.map