"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var gitdiff_parser_1 = __importDefault(require("gitdiff-parser"));
var zipChanges = function (changes) {
    var result = changes.reduce(function (_a, current, i) {
        var result = _a[0], last = _a[1], lastDeletionIndex = _a[2];
        if (!last) {
            result.push(current);
            return [result, current, current.isDelete ? i : -1];
        }
        if (current.isInsert && lastDeletionIndex >= 0) {
            result.splice(lastDeletionIndex + 1, 0, current);
            // The new `lastDeletionIndex` may be out of range, but `splice` will fix it
            return [result, current, lastDeletionIndex + 2];
        }
        result.push(current);
        // Keep the `lastDeletionIndex` if there are lines of deletions,
        // otherwise update it to the new deletion line
        var newLastDeletionIndex = current.isDelete ? (last.isDelete ? lastDeletionIndex : i) : i;
        return [result, current, newLastDeletionIndex];
    }, [[], null, -1])[0];
    return result;
};
var mapHunk = function (hunk, options) {
    var changes = options.nearbySequences === 'zip' ? zipChanges(hunk.changes) : hunk.changes;
    return __assign(__assign({}, hunk), { isPlain: false, changes: changes });
};
var mapFile = function (file, options) {
    var hunks = file.hunks.map(function (hunk) { return mapHunk(hunk, options); });
    return __assign(__assign({}, file), { hunks: hunks });
};
var normalizeDiffText = function (text) {
    if (text.indexOf('diff --git') === 0) {
        return text;
    }
    var indexOfFirstLineBreak = text.indexOf('\n');
    var indexOfSecondLineBreak = text.indexOf('\n', indexOfFirstLineBreak + 1);
    var firstLine = text.slice(0, indexOfFirstLineBreak);
    var secondLine = text.slice(indexOfFirstLineBreak + 1, indexOfSecondLineBreak);
    var oldPath = firstLine.slice(4);
    var newPath = secondLine.slice(4);
    var segments = ["diff --git " + oldPath + " " + newPath, 'index 1111111..2222222 100644', text];
    return segments.join('\n');
};
exports.parseDiff = function (text, options) {
    if (options === void 0) { options = {}; }
    var diffText = normalizeDiffText(text);
    var files = gitdiff_parser_1.default.parse(diffText);
    return files.map(function (file) { return mapFile(file, options); });
};
//# sourceMappingURL=index.js.map